use shiny_std::arena::intern;

use crate::{
    bin, int,
    ptr::{self, TyTag as Tag},
};

#[derive(Clone)]
pub struct Ctx {
    /// Product types.
    prod: intern::Arena<bin::Prod, ptr::RawTy>,
    /// Sum types.
    sum: intern::Arena<bin::Sum, ptr::RawTy>,
    /// Integers.
    ints: intern::Arena<int::Int, ptr::RawTy>,
}

impl Ctx {
    pub fn int(&self, ptr: ptr::RawTy) -> Option<int::Int> {
        self.ints.get(ptr).cloned()
    }

    pub fn prod(&self, ptr: ptr::RawTy) -> Option<bin::Prod> {
        self.prod.get(ptr).cloned()
    }

    pub fn sum(&self, ptr: ptr::RawTy) -> Option<bin::Sum> {
        self.sum.get(ptr).cloned()
    }
}

impl Ctx {
    pub fn mk_int(&mut self, signedness: int::Signedness) -> ptr::Ty {
        let ptr = self.ints.insert(int::Int(signedness));
        ptr::Ty::new(Tag::Int, ptr)
    }

    pub fn mk_sum(&mut self, lhs: ptr::Ty, rhs: ptr::Ty) -> ptr::Ty {
        let ptr = self.sum.insert(bin::Sum { lhs, rhs });
        ptr::Ty::new(Tag::Sum, ptr)
    }

    pub fn mk_prod(&mut self, lhs: ptr::Ty, rhs: ptr::Ty) -> ptr::Ty {
        let ptr = self.prod.insert(bin::Prod { lhs, rhs });
        ptr::Ty::new(Tag::Prod, ptr)
    }
}
