//! # Pointers to needed information.
//!
//! If values are equal, then their pointers are equal.
//! This means that there's unambiguous mapping from id to value and
//! back.

use shiny_std::arena::ptr::{define, tags};

/// Type.
pub type Ty = Tagged<TyTag, RawTy>;

#[derive(Debug, Clone, Copy, PartialEq, Eq, PartialOrd, Ord, Hash)]
pub struct Tagged<T, P> {
    pub tag: T,
    pub ptr: P,
}

impl<T, P> Tagged<T, P> {
    pub const fn new(tag: T, ptr: P) -> Self {
        Self { tag, ptr }
    }
}

tags! {
    pub enum TyTag {
        Prod,
        Sum,
        Int,
        Str,
    }
}

define! {
    /// Untagged pointer to the type.
    pub struct RawTy;

    /// Pointer to ID.
    pub struct Id;

    /// Pointer to a pattern.
    pub struct Pat;

    /// Pointer to a *complete* value.
    pub struct Value;
}
